package ssw.mj.symtab;

/**
 * MicroJava Symbol Table Objects: Every named object in a program is stored in
 * an <code>Obj</code> node. Every scope has a list of objects declared within
 * it.
 */
public class Obj {
	/** Possible codes for object kinds. */
	public enum Kind {
		Con, Var, Type, Meth, Prog
	}

	/** Kind of the object node. */
	public Kind kind;
	/** Name of the object node. */
	public String name;
	/** Type of the object node. */
	public Struct type;
	/** Next element of the linked Obj list in this scope. */
	public Obj next;
	/** Only for Con: Value of the constant. */
	public int val;
	/** Only for Var, Meth: Offset of the element. */
	public int adr;
	/** Only for Var: Declaration level (0..global, 1..local) */
	public int level;
	/** Only for Meth: Number of parameters. */
	public int nPars;
	/** Only for Meth / Prog: List of local variables / global declarations. */
	public Obj locals;

	public Obj(Kind kind, String name, Struct type) {
		this.kind = kind;
		this.name = name;
		this.type = type;
	}

	/**
	 * String representation for debugging.
	 */
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		dump(sb, "", false, false);
		return sb.toString();
	}

	/**
	 * String representation for JUnit test cases.
	 */
	public void dump(StringBuilder sb, String indent, boolean isField,
			boolean follow) {
		sb.append(indent);

		switch (kind) {
			case Con:
				sb.append("Constant: ");
				if (type != null) {
					type.dump(sb, indent, false);
				}
				sb.append(" ").append(name).append(" = ");
				if (type == Tab.charType) {
					sb.append("'").append((char) val).append("'");
				} else {
					sb.append(val);
				}
				break;
			case Var:
				if (isField) {
					sb.append("Field ");
				} else if (level == 0) {
					sb.append("Global Variable ");
				} else {
					sb.append("Local Variable ");
				}
				sb.append(adr).append(": ");
				if (type != null) {
					type.dump(sb, indent, false);
				}
				sb.append(" ").append(name);
				break;
			case Type:
				sb.append("Type ").append(name).append(": ");
				if (type != null) {
					type.dump(sb, indent + "  ", true);
				}
				break;
			case Meth:
				sb.append("Method: ");
				if (type != null) {
					type.dump(sb, indent, false);
				}
				sb.append(" ").append(name).append("(").append(nPars);
				sb.append(") at ").append(adr);
				break;
			case Prog:
				sb.append("Program ").append(name).append(":");
				break;
		}

		if (follow && locals != null) {
			sb.append("\n");
			locals.dump(sb, indent + "  ", isField, follow);
		}
		if (follow && next != null) {
			sb.append("\n");
			next.dump(sb, indent, isField, follow);
		}
	}
}
